/* ===================================================================
 * Auto power control of Stereo Amplifier
 * ===================================================================
 * This application is used turn Stereo Amplifier OFF if no music is
 * played. It also turns amplifier ON if the source is detected to play
 * while the amplifier is detected as OFF.
 *
 * Power Control:
 * --------------
 *
 * What if I have different equipment?
 * -----------------------------------
 * 1. Learn the IR codes
 *		You need to learn the IR codes of your amplifier. LIRC is a good
 * 		source for such information.
 * 2. Change the IR Commands in constants below
 *		If your amplifier is also based on NEC IR controller then it will
 *		be extremely easy to swap codes and that's it. If however your
 *		amplifier is using other sort of IR, you will need to change
 *		the calling to the IRRemote library accordingly
 *		I placed a comment like this:
 *         // CHANGE the below if your amplifier is not based on NEC
 *		In every place I believe need to be modified
 * 3. Compile and load new code to your Arduino
 *
 * ===================================================================
 * By: Zakie Mashiah
 *		You can copy, distribute and do whatever you want with this code.
 * Using IRemote Arduino library by Ken Shirriff (GNU License)
 * 		see http://arcfn.com/2009/08/multi-protocol-infrared-remote-library.html
 *		for details about the library
 * ===================================================================
 */ 
#include <avr/sleep.h>
#include <WProgram.h>
#include <IRremote.h>

#include "Amplifier.h"

static IRsend irsend;
 
 /* Amplifier Class
  * Preseves the state, operate (control) the Amplifier through IR
  */

// Initializes the class, max_idle is in minutes
void AmplifierState::Init(unsigned long offCommand, unsigned long onCommand, unsigned long onInput, unsigned long max_idle)
{
	offIRCommand 	  	= offCommand;
	idleTimeToTurnOff 	= max_idle;
	onIRCommand 		= onCommand;
	onIRInput 			= onInput;

	Serial.print("Amp. commands:\n\tOFF\t");	Serial.print(offIRCommand, HEX);
	Serial.print("\n\tON\t");					Serial.print(onIRCommand, HEX);
	Serial.print("\n\tCOAX1\t"); 				Serial.print(onIRInput, HEX);
	Serial.print("\n\tIdle time: "); 			Serial.println(idleTimeToTurnOff, DEC);
}

// Should be called on every sample of the audio signal. mp is true if music detected
void AmplifierState::SetAmplifierSample(bool mp)
{
	unsigned long nowM = millis(); // Take current time
	unsigned long diff;
	
	if (mp)
	{
		// Music is playing, act accordingly
		isAmplifierOn = true;
		isNowPlaying = true;
		lastIdle = 0;
	}
	else
	{
		// No music is playing
		if (isAmplifierOn)
		{
			isNowPlaying = false;
			if (lastIdle != 0)
			{
                static unsigned long secondsLeftLast = 0;
				unsigned int  secondsLeft;
				
				diff = 0;
				if (nowM < lastIdle) // Overflow of millis()
					diff = nowM + (0xFFFFFFFFUL - lastIdle);
				else // Standard difference between millis reading
					diff = nowM - lastIdle;
				
				secondsLeft = (int)((idleTimeToTurnOff - diff)/1000UL);
				if (secondsLeft != secondsLeftLast) // Don't fill the terminal with garbage, only once a second
				{
					secondsLeftLast = secondsLeft;
					Serial.print("Idle for ");
					Serial.print(diff, DEC);
					Serial.print("\t (left)\t");
					Serial.println(secondsLeft, DEC);
				}
	
				if (diff > idleTimeToTurnOff)
				{
					isAmplifierOn = false;
					// Turn off the Amplifier, call twice just in case
					// CHANGE the below if your amplifier is not based on NEC
					irsend.sendNEC(offIRCommand, 32);
					irsend.sendNEC(offIRCommand, 32);
					Serial.println(">>> Amplifier turned off <<<");
				}
			}
			else
				lastIdle = nowM; // Now is first time we see idle, save the time.
		}
	}
}

// Sends STBY_ON code (x2) and Input selection (Coax1) (x2)
void AmplifierState::TurnOn()
{
	if (isAmplifierOn == false)
	{
		Serial.println(">>> Amplifier turned ON <<");
		isAmplifierOn = true;
		// CHANGE the below if your amplifier is not based on NEC IR Protocol
		// Sending every command twice, just in case...
		irsend.sendNEC(onIRCommand, 32);
		irsend.sendNEC(onIRCommand, 32);
		delay(500);	// Wait for amplifier to turn on and send selection for input source
		irsend.sendNEC(onIRInput, 32);
		irsend.sendNEC(onIRInput, 32);
		lastIdle = 0;
	}
}


/* ===================================================================
 * AnalogChannel Class
 * Represent a Left+Right Channels to sample
 * ===================================================================
 */

AnalogChannel::AnalogChannel()
{
	leftInput   = 0;
	rightInput  = 0;
	maxLeft     = 0;
	maxRight    = 0;
	channelName = NULL;
}

void AnalogChannel::Init(int leftPin, int rightPin, char *name)
{
	leftInput = leftPin;
	rightInput = rightPin;
	channelName = name;
}

#define ANALOG_SAMPLES 20
const int analogThreashold = 5; // Less than this is noise
int AnalogChannel:: _ReadInput(int pinL, int pinR)
{
  	int maxRead = 0;
	int samplesR[ANALOG_SAMPLES];
	int samplesL[ANALOG_SAMPLES];
	int i, n;

	n = sizeof(samplesL)/sizeof(samplesL[0]);
	
	// Take samples on right channel
	for(i=0; i < n; i++)
	{
		samplesL[i] = analogRead(pinL);
		samplesR[i] = analogRead(pinR);
		delay(2 * i);	// This is pseudo random delay compared to music
	}
	for(i=0; i < n; i++)
		if (samplesR[i] > samplesL[i])
		{
			if (samplesR[i] > maxRead)
				maxRead = samplesR[i];
		}
		else
			if (samplesL[i] > maxRead)
				maxRead = samplesL[i];

	return maxRead;
}
	
bool AnalogChannel::IsMusicPlaying()
{
	int maxRead;

	Serial.print("AnChan:");

	if (channelName != NULL)
	{
		Serial.print(" CH [");
		Serial.print(channelName);
		Serial.print("] ");
	}

	// Take samples on left and right channel
	maxRead = _ReadInput(leftInput, rightInput);
	Serial.print("\tTotal=");
	Serial.println(maxRead, DEC);

	return (maxRead > analogThreashold); // should be maxRead > ...
}