/* ===================================================================
 * Stereo Amplifier control for Kenwood 6030
 * ===================================================================
 * Module     : KenwoodIdle.pde
 * ===================================================================
 * By:	Zakie Mashiah
 * ===================================================================
 * Edits:
 *	Nov-2011	Zakie Mashiah   Use Arduino functions for I/O and
 *								pin setting. Move to CPP
 *	Feb-2012	Zakie Mashih	Support the v0.95 of the shield
 *  May-2012    Zakie Mashiah   Use for Kenwood amplifier without
 *                              Spectrum analyzer (no need)
 * ===================================================================
*/
#include <IRremote.h>

#include "Amplifier.h" // Requires IRremote.h
#include "ledNotifier.h"


/* ===========================================================================
 * Constants
 * =========================================================================== */

// maxIdleMinutes - Longer than this, amplifier will be turned off
#define maxIdleMinutes 5

/* ------------------------
 * IR Commands
 * ------------------------
 * The discrete IR commands played by the circuit to control 
 * PeachTree Nova Amplifier
 * CHANGE these according to your amplifier
 */
const unsigned long kenwoodPower   = 0x1DE2B946; // The IR command to turn off the Nova
const unsigned long kenwoodInputRCode = 0x00000A956; // The IR command to change input to Coax1 

const int cycleLedPin = 13;


/* ===========================================================================
 * 							Global Variables
 * =========================================================================== */
static AnalogChannel    chSonos;					// Sampler on analog inputs where Sonos is connected
static AnalogChannel    chAmplifier;                // Sampler on analog inputs where Amplifier monitor is connected
static AmplifierState   kenwoodAmplifier;			// Representing the Amplifier object and state machine
static LedNotifier      ledNotifier(9, 10, 11);		// RGB Led controller based on state


/* ===========================================================================
 * 							Setup section
 * =========================================================================== */
 void setup()
{
    Serial.begin(9600);
    Serial.print("Starting...\n");
 	
	kenwoodAmplifier.Init(	kenwoodPower, kenwoodPower, kenwoodInputRCode,
						(unsigned long)(maxIdleMinutes * 60L * 1000L));
	chSonos.Init(A2, A3, "Sonos");
    chAmplifier.Init(A4, A5, "Amp");

    ledNotifier.Init(9, 10, 11);
    pinMode(cycleLedPin, OUTPUT);
	Serial.println("setup done.");
}


/* ===========================================================================
 * 							Main loop section
 * =========================================================================== */
void DisplayStatus() // Fancy status display
{
	char buff[50];
	static unsigned long last = 0;
	unsigned long _now = millis();
	
	if ((_now - last) > 1000)
	{
		// Display status on terminal only once a second
		last = _now;
		sprintf(buff, "Sys status:\tPlay=%d\tIdle=%d\tAmp=%d\tSrc=%d",
			kenwoodAmplifier.IsNowPlaying(), kenwoodAmplifier.IsIdle(), kenwoodAmplifier.IsAmplifierOn(),
			ledNotifier.isSourceOn() );
		Serial.println(buff);
		if (kenwoodAmplifier.IsAmplifierOn() == false)
			ledNotifier.SetAmpState(false);	// if the Amplifier turned itself off, tell the LED manager about it
	}
	ledNotifier.Display();
}


void loop()
{
	int musicSample = 0;

    digitalWrite(cycleLedPin, true);
	
    musicSample = chAmplifier.IsMusicPlaying();
	kenwoodAmplifier.SetAmplifierSample(musicSample != 0);	// Tell the amplifier object what the state is
	ledNotifier.SetMusicState(musicSample != 0);		// And the LED notifier object

	if (kenwoodAmplifier.IsAmplifierOn() == false) // No need to sample Sonos if amplifier is on
	{
		// Amplifier is off, do check to see the Sonos / source input has signal
		if (chSonos.IsMusicPlaying())
		{
			// Amplifier is OFF but Sonos does play, so turn amplifier ON
			ledNotifier.SourcePlaying();
			kenwoodAmplifier.TurnOn();
			ledNotifier.SetAmpState(true); // Since we turned amplifier ON, show it on LEDs
		}
		else
			ledNotifier.SourceOff();
	}
	else
		ledNotifier.SetAmpState(true);

	DisplayStatus();

    digitalWrite(cycleLedPin, false);
}
